import psycopg
import re
from pystac import Catalog
from geovisio import stac

from . import conftest


def test_landing(client):
    response = client.get("/api/")
    data = response.json

    assert response.status_code == 200
    assert data["type"] == "Catalog"
    ctl = Catalog.from_dict(data)
    assert len(ctl.links) > 0
    assert ctl.title == "GeoVisio STAC API"
    assert ctl.id == "geovisio"
    assert ctl.extra_fields.get("extent") is None
    assert ctl.get_links("self")[0].get_absolute_href() == "http://localhost:5000/api/"
    assert ctl.get_links("xyz")[0].get_absolute_href() == "http://localhost:5000/api/map/{z}/{x}/{y}.mvt"
    assert ctl.get_links("collection-preview")[0].get_absolute_href() == "http://localhost:5000/api/collections/{id}/thumb.jpg"
    assert ctl.get_links("item-preview")[0].get_absolute_href() == "http://localhost:5000/api/pictures/{id}/thumb.jpg"
    assert ctl.get_links("data", "application/json")[0].get_absolute_href() == "http://localhost:5000/api/collections"
    assert ctl.get_links("data", "application/rss+xml")[0].get_absolute_href() == "http://localhost:5000/api/collections?format=rss"


@conftest.SEQ_IMGS
def test_landing_extent(datafiles, initSequence):
    client = initSequence(datafiles, preprocess=False)
    response = client.get("/api/")
    data = response.json

    assert response.status_code == 200
    assert data["type"] == "Catalog"
    ctl = Catalog.from_dict(data)
    assert len(ctl.links) > 0

    assert len(ctl.extra_fields["extent"]["temporal"]["interval"]) == 1
    assert len(ctl.extra_fields["extent"]["temporal"]["interval"][0]) == 2
    assert re.match(r"^2021-07-29T", ctl.extra_fields["extent"]["temporal"]["interval"][0][0])
    assert re.match(r"^2021-07-29T", ctl.extra_fields["extent"]["temporal"]["interval"][0][1])
    assert ctl.extra_fields["extent"]["spatial"] == {
        "bbox": [[1.9191854000091553, 49.00688934326172, 1.919199824333191, 49.00697708129883]]
    }


def test_conformance(client):
    response = client.get("/api/conformance")
    data = response.json

    assert response.status_code == 200
    assert data["conformsTo"] == stac.CONFORMANCE_LIST


def test_no_license_main_endpoint(no_license_app_client):
    response = no_license_app_client.get("/api")
    assert response.status_code < 400

    # there should not be a license link since we do not know the license
    rels = [l for l in response.json["links"] if l["rel"] == "license"]
    assert not rels


def test_defined_license_main_endpoint(client):
    response = client.get("/api")
    assert response.status_code < 400

    # there should not be a license link since we do not know the license
    rels = [l for l in response.json["links"] if l["rel"] == "license"]
    assert len(rels) == 1
    l = rels[0]
    assert l == {
        "href": "https://raw.githubusercontent.com/DISIC/politique-de-contribution-open-source/master/LICENSE",
        "rel": "license",
        "title": "License for this object (etalab-2.0)",
    }


@conftest.SEQ_IMGS
def test_user_catalog(datafiles, dburl, initSequence):
    client = initSequence(datafiles, preprocess=False)

    # Get user ID
    with psycopg.connect(dburl) as conn:
        with conn.cursor() as cursor:
            userId, userName = cursor.execute("SELECT id, name FROM accounts WHERE is_default").fetchone()

            response = client.get("/api/users/" + str(userId) + "/catalog")
            data = response.json

            assert response.status_code == 200
            assert data["type"] == "Catalog"
            ctl = Catalog.from_dict(data)
            assert len(ctl.links) > 0
            assert ctl.title == userName + "'s sequences"
            assert ctl.id == f"user:{userId}"
            assert ctl.description == "List of all sequences of user " + userName
            assert ctl.extra_fields.get("extent") is None
            assert ctl.get_links("self")[0].get_absolute_href() == "http://localhost/api/users/" + str(userId) + "/catalog/"

            # Check links
            for link in ctl.get_links("child"):
                assert link.title is not None
                assert link.extra_fields["id"] is not None
                assert link.get_absolute_href() == "http://localhost/api/collections/" + link.extra_fields["id"]
                assert link.extra_fields["extent"]["temporal"] == {"interval": [["2021-07-29T09:16:54+00:00", "2021-07-29T09:17:02+00:00"]]}
                assert link.extra_fields["stats:items"]["count"] == 5
